/*
* I2C sample for IO-Warrior56
* This sample use the first found IO-Warrior device on IowKitOpenDevice().
* The I2C device (address) which used in this sample may be changed for your device.
*/

#include <stdio.h>
#include <string.h>
#include "iowkit.h"


//Print out the report bytes
void ShowReport(IOWKIT56_SPECIAL_REPORT data, ULONG count, const char* text)
{
    if(count > IOWKIT56_SPECIAL_REPORT_SIZE)
        count = IOWKIT56_SPECIAL_REPORT_SIZE;

    printf("%s: ", text);
    for(ULONG i=0; i<count; i++)
        printf("%02X ", data.Bytes[i]);

    printf("\n");
}


int main(int argc, char *argv[])
{
    IOWKIT_HANDLE handle;
    IOWKIT56_SPECIAL_REPORT report;

    // Print version of SDK/API/DLL
    printf("API: %s\n", IowKitVersion());

    // Open device and use first founc device furhter
    handle = IowKitOpenDevice();
    if (handle == NULL)
    {
        printf("Failed to open device\n");
        return 0;
    }

    //Allow only an IO-Warrior56 to run programm
    if(IowKitGetProductId(handle) != IOWKIT_PID_IOW56)
    {
        printf("No IO-Warrior56 found\n");
        IowKitCloseDevice(handle);
        return 0;
    }

    //Set timeouts for write and read. default is INFINITE
    IowKitSetWriteTimeout(handle, 1000);
    IowKitSetTimeout(handle, 1000);

    //Enable I2C mode
    memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);

    report.ReportID = 0x01;     //Special mode: I2C
    report.Bytes[0] = 0x01;     //Enable I2C mode
    report.Bytes[1] = 0x00;     //Flags: pullups ON, speed 93.75kHz

    IowKitWrite(handle, IOW_PIPE_SPECIAL_MODE, (char *)&report, IOWKIT56_SPECIAL_REPORT_SIZE);

    //Search for I2C devices by reading 1 byte and print out (no further use of i2c address)
    for (int i = 1; i < 127; i++)
    {
        memset(&report, 0, IOWKIT56_SPECIAL_REPORT_SIZE);
        report.ReportID = 0x03;             //I2C read
        report.Bytes[0] = 0x01;             //Read one byte
        report.Bytes[1] = (i << 1) | 0x01;  //Device address (8 bit + read bit)

        if (IowKitWrite(handle, IOW_PIPE_SPECIAL_MODE, (char *)&report, IOWKIT56_SPECIAL_REPORT_SIZE) == IOWKIT56_SPECIAL_REPORT_SIZE)
        {
            if (IowKitRead(handle, IOW_PIPE_SPECIAL_MODE, (char *)&report, IOWKIT56_SPECIAL_REPORT_SIZE) == IOWKIT56_SPECIAL_REPORT_SIZE)
            {
                if (report.Bytes[0] == 0x01) //Read byte 0, if byte count = 1 -> device present
                    printf("I2C device found (8Bit): 0x%02X\n", (i << 1));
            }
            else
            {
                printf("ERROR on IowKitRead(). CANCEL\n");
                break;
            }
        }
        else
        {
            printf("ERROR on IowKitWrite(). CANCEL\n");
            break;
        }
    }

    //Write to I2C device
    memset(&report, 0x00, IOWKIT56_SPECIAL_REPORT_SIZE);
    report.ReportID = 0x02;     //I2C write
    report.Bytes[0] = 0xC2;     //Flags + byte count
    report.Bytes[1] = 0x88;     //I2C slave address 8Bit (0x88 for sample)
    report.Bytes[2] = 0x00;     //I2C slave register / first data byte
    //report.Bytes[3] = 0x00;   //more data.....

    printf("Write to I2C device...\n");

    if (IowKitWrite(handle, IOW_PIPE_SPECIAL_MODE, (char *)&report, IOWKIT56_SPECIAL_REPORT_SIZE) == IOWKIT56_SPECIAL_REPORT_SIZE)
    {
        ShowReport(report, (report.Bytes[0] & 0x3F), "Write");

        if (IowKitRead(handle, IOW_PIPE_SPECIAL_MODE, (char *)&report, IOWKIT56_SPECIAL_REPORT_SIZE) == IOWKIT56_SPECIAL_REPORT_SIZE)
            ShowReport(report, (report.Bytes[0] & 0x3F), "Read");
        else
            printf("ERROR on IowKitRead()\n");
    }
    else
        printf("ERROR on IowKitWrite()\n");

    //Read from I2C device
    int count = 6;
    memset(&report, 0x00, IOWKIT56_SPECIAL_REPORT_SIZE);
    report.ReportID = 0x03;         //I2C read
    report.Bytes[0] = count;        //data count
    report.Bytes[1] = 0x88 | 0x01;  //I2C slave address + read bit (8 bit) (0x88 for sample)
    
    printf("Read from I2C device...\n");

    if (IowKitWrite(handle, IOW_PIPE_SPECIAL_MODE, (char *)&report, IOWKIT56_SPECIAL_REPORT_SIZE) == IOWKIT56_SPECIAL_REPORT_SIZE)
    {
        ShowReport(report, 2, "Write");
        
        if (IowKitRead(handle, IOW_PIPE_SPECIAL_MODE, (char *)&report, IOWKIT56_SPECIAL_REPORT_SIZE) == IOWKIT56_SPECIAL_REPORT_SIZE)
        {
            printf("Read: ");
            for(int i=0; i<count; i++)
                printf("%02X ", report.Bytes[i]);

            printf("\n");
        }
        else
            printf("ERROR on IowKitRead()\n");
    }
    else
        printf("ERROR on IowKitWrite()\n");

    //Disable I2C mode (free IO-Pins)
    memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);

    report.ReportID = 0x01;     //Special mode: I2C
    report.Bytes[0] = 0x00;     //Disable I2C mode
    report.Bytes[1] = 0x00;     //Flags: pullups ON, speed 93.75kHz

    IowKitWrite(handle, IOW_PIPE_SPECIAL_MODE, (char *)&report, IOWKIT56_SPECIAL_REPORT_SIZE);

    // Close device
    IowKitCloseDevice(handle);

    return 0;
}